package java.time.chrono;

import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.format.ResolverStyle;
import java.time.temporal.TemporalField;
import java.util.Map;
import java.time.temporal.ValueRange;
import java.time.temporal.ChronoField;
import java.util.Arrays;
import java.util.List;
import java.time.Instant;
import java.time.temporal.TemporalAccessor;
import java.time.ZoneId;
import java.time.Clock;
import java.time.LocalDate;
import java.io.Serializable;

public final class MinguoChronology extends AbstractChronology implements Serializable
{
    public static final MinguoChronology INSTANCE;
    private static final long serialVersionUID = 1039765215346859963L;
    static final int YEARS_DIFFERENCE = 1911;
    
    private MinguoChronology() {
    }
    
    @Override
    public String getId() {
        return "Minguo";
    }
    
    @Override
    public String getCalendarType() {
        return "roc";
    }
    
    @Override
    public MinguoDate date(Era era, int n, int n2, int n3) {
        return this.date(this.prolepticYear(era, n), n2, n3);
    }
    
    @Override
    public MinguoDate date(int n, int month, int dayOfMonth) {
        return new MinguoDate(LocalDate.of(n + 1911, month, dayOfMonth));
    }
    
    @Override
    public MinguoDate dateYearDay(Era era, int n, int n2) {
        return this.dateYearDay(this.prolepticYear(era, n), n2);
    }
    
    @Override
    public MinguoDate dateYearDay(int n, int dayOfYear) {
        return new MinguoDate(LocalDate.ofYearDay(n + 1911, dayOfYear));
    }
    
    @Override
    public MinguoDate dateEpochDay(long epochDay) {
        return new MinguoDate(LocalDate.ofEpochDay(epochDay));
    }
    
    @Override
    public MinguoDate dateNow() {
        return this.dateNow(Clock.systemDefaultZone());
    }
    
    @Override
    public MinguoDate dateNow(ZoneId zone) {
        return this.dateNow(Clock.system(zone));
    }
    
    @Override
    public MinguoDate dateNow(Clock clock) {
        return this.date(LocalDate.now(clock));
    }
    
    @Override
    public MinguoDate date(TemporalAccessor temporal) {
        if ((temporal instanceof MinguoDate)) {
            return (MinguoDate)temporal;
        }
        return new MinguoDate(LocalDate.from(temporal));
    }
    
    @Override
    public ChronoLocalDateTime<MinguoDate> localDateTime(TemporalAccessor temporalAccessor) {
        return super.localDateTime(temporalAccessor);
    }
    
    @Override
    public ChronoZonedDateTime<MinguoDate> zonedDateTime(TemporalAccessor temporalAccessor) {
        return super.zonedDateTime(temporalAccessor);
    }
    
    @Override
    public ChronoZonedDateTime<MinguoDate> zonedDateTime(Instant instant, ZoneId zoneId) {
        return super.zonedDateTime(instant, zoneId);
    }
    
    @Override
    public boolean isLeapYear(long n) {
        return IsoChronology.INSTANCE.isLeapYear(n + 1911L);
    }
    
    @Override
    public int prolepticYear(Era era, int n) {
        if (!(era instanceof MinguoEra)) {
            throw new ClassCastException("Era must be MinguoEra");
        }
        return (era == MinguoEra.ROC) ? n : (1 - n);
    }
    
    @Override
    public MinguoEra eraOf(int n) {
        return MinguoEra.of(n);
    }
    
    @Override
    public List<Era> eras() {
        return Arrays.asList(MinguoEra.values());
    }
    
    @Override
    public ValueRange range(ChronoField chronoField) {
        switch (chronoField) {
            case PROLEPTIC_MONTH: {
                ValueRange range = ChronoField.PROLEPTIC_MONTH.range();
                return ValueRange.of(range.getMinimum() - 22932L, range.getMaximum() - 22932L);
            }
            case YEAR_OF_ERA: {
                ValueRange range2 = ChronoField.YEAR.range();
                return ValueRange.of(1L, range2.getMaximum() - 1911L, -range2.getMinimum() + 1L + 1911L);
            }
            case YEAR: {
                ValueRange range3 = ChronoField.YEAR.range();
                return ValueRange.of(range3.getMinimum() - 1911L, range3.getMaximum() - 1911L);
            }
            default: {
                return chronoField.range();
            }
        }
    }
    
    @Override
    public MinguoDate resolveDate(Map<TemporalField, Long> map, ResolverStyle resolverStyle) {
        return (MinguoDate)super.resolveDate(map, resolverStyle);
    }
    
    @Override
    Object writeReplace() {
        return super.writeReplace();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    static {
        INSTANCE = new MinguoChronology();
    }
}
